"""constants.py Stores constants such as number of nodes NNODES etc."""
NNODES_ov_RLL10deg_CSne4 = 683
NNODES_outCSne8 = 386
NNODES_outCSne30 = 5402
NNODES_outRLL1deg = 64442
DATAVARS_outCSne30 = 2
TRI_AREA = 1.047
# 4*Pi is 12.56
MESH30_AREA = 12.566
PSI_INTG = 12.566
VAR2_INTG = 12.566
