from warnings import warn


def _read_shpfile(filepath):
    """Read shape file.

    Not implemented.
    """
    raise RuntimeError(
        "Function not implemented yet. FYI, attempted to read SHAPE file: " +
        str(filepath))
    # TODO: create ds
